from sup import *
from unsup import *


def eval_parse(parser):
    parser.add_argument('-af', '--args_file',
                        help='file to load arguments from', type=str)


def main():
    parser = parse()
    eval_parse(parser)
    sup_parse(parser)
    unsup_parse(parser)
    args = parser.parse_args()
    eta = args.eta
    niter = args.niter
    batch_size = args.batch_size
    print(args)
    device = torch.device('cuda:'+str(args.cuda))
    if args.seed >= 0:
        torch.manual_seed(args.seed)

    load_prefix = args.load_prefix
    args_file = args.args_file
    if load_prefix:
        args = parser.parse_args(['@'+args_file])
        print(args)

    (tr_sents, tr_labels), (de_sents, de_labels), (te_sents, te_labels) =\
        load_dataset(args.dataset, shuffle=True, dev=True)

    if args.verbose:
        print('Initialize model')
    if args.gru:
        model = GRUEncoder(input_dimension=args.input_dimension,
                           output_dimension=args.output_dimension,
                           max_length=args.max_length, dropout=args.dropout,
                           freeze=args.freeze, vocabfile=args.words,
                           min_count=args.min_count, vectorfile=args.pretrained)
    else:
        model = Encoder(input_dimension=args.output_dimension,
                        max_length=args.max_length, freeze=args.freeze,
                        vocabfile=args.words, min_count=args.min_count,
                        vectorfile=args.pretrained)

    model.load_state_dict(torch.load(load_prefix))
    n_class = len(np.unique(tr_labels))
    clf = nn.Linear(args.output_dimension, n_class, bias=False)
    model.to(device)
    clf.to(device)

    tr_x = model.str2long(tr_sents, device=device)
    de_x = model.str2long(de_sents, device=device)
    te_x = model.str2long(te_sents, device=device)
    tr_y = Variable(torch.LongTensor(tr_labels)).to(device)
    de_y = Variable(torch.LongTensor(de_labels)).to(device)
    te_y = Variable(torch.LongTensor(te_labels)).to(device)
    data = ((tr_x, tr_y), (te_x, te_y), (de_x, de_y))

    tr_x_np = transform_(model, tr_x, 100)
    te_x_np = transform_(model, te_x, 100)
    tr_y_np = tr_y.detach().cpu().numpy()
    mean_clf = torch.from_numpy(np.vstack([tr_x_np[tr_y_np==i].mean(0) for i in range(n_class)]))
    clf.weight.data.copy_(mean_clf)

    tr_x_ = Variable(torch.Tensor(tr_x_np)).to(device)
    te_x_ = Variable(torch.Tensor(te_x_np)).to(device)
    criterion = nn.CrossEntropyLoss()  
    optimizer = torch.optim.SGD(clf.parameters(), lr=eta)  

    # Training the Model
    n_tr = tr_y.shape[0]
    total_loss = 0.0
    total_data = 0
    print('\rEpoch %d done. Accuracies: '%(0), eval_multiclass(data[:2], model, clf=clf, k=10))
    for epoch in range(niter):
        shuffle = np.random.permutation(list(range(n_tr)))
        for i in range(0, n_tr, batch_size):
            batch_idx = shuffle[i:i+batch_size]
            batch_x = tr_x_[batch_idx]
            batch_y = tr_y[batch_idx]
            
            # Forward + Backward + Optimize
            optimizer.zero_grad()
            batch_pred = clf(batch_x)
            loss = criterion(batch_pred, batch_y) / np.log(n_class)
            loss.backward()
            optimizer.step()
            total_loss += float(loss)*len(batch_idx)
            total_data += len(batch_idx)
            
            if (i//batch_size) % 100 == 0:
                print ('\r\tEpoch: [%d/%d], Step: [%d/%d], Loss: %.4f' 
                       % (epoch+1, niter, (i+1)//batch_size, n_tr//batch_size, total_loss/total_data), end='')
        if (epoch+1) % 5 == 0:
            print('\rEpoch %d done. Accuracies: '%(epoch+1), eval_multiclass(data[:2], model, clf=clf, k=10))


if __name__ == '__main__': main()
